# -*- coding: utf-8 -*-
#
# Copyright (C) 2019-2024 CERN.
# Copyright (C) 2019-2020 Northwestern University.
# Copyright (C)      2021 TU Wien.
# Copyright (C) 2024 Graz University of Technology.
#
# Invenio App RDM is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Filters to be used in the Jinja templates."""

from os.path import splitext

import idutils
from babel.numbers import format_compact_decimal, format_decimal
from flask import current_app, url_for
from invenio_base.utils import obj_or_import_string
from invenio_i18n import get_locale
from invenio_previewer.views import is_previewable
from invenio_records_files.api import FileObject
from invenio_records_permissions.policies import get_record_permission_policy

from ..previewer.iiif_simple import previewable_extensions as image_extensions


def make_files_preview_compatible(files):
    """Processes a list of RecordFiles to a list of FileObjects.

    This is needed to make the objects compatible with invenio-previewer.
    """
    file_objects = []
    for file in files:
        file_objects.append(FileObject(obj=files[file].object_version, data={}).dumps())
    return file_objects


def select_preview_file(files, default_preview=None):
    """Return file to preview or None if no previewable file."""
    selected = None
    for f in files or []:
        file_type = splitext(f.get("key", ""))[1][1:].lower()
        if is_previewable(file_type):
            if selected is None:
                selected = f
            elif f.get("key") == default_preview:
                return f
    return selected


def to_previewer_files(record):
    """Get previewer-compatible files list."""
    return [
        FileObject(obj=f.object_version, data=f.metadata or {})
        for f in record.files.values()
    ]


def can_list_files(record):
    """Permission check if current user can list files of record.

    The current_user is used under the hood by flask-principal.

    Once we move to Single-Page-App approach, we likely want to enforce
    permissions at the final serialization level (only).
    """
    PermissionPolicy = get_record_permission_policy()
    return PermissionPolicy(action="read_files", record=record).can()


def pid_url(identifier, scheme=None, url_scheme="https"):
    """Convert persistent identifier into a link."""
    if scheme is None:
        try:
            scheme = idutils.detect_identifier_schemes(identifier)[0]
        except IndexError:
            scheme = None
    try:
        if scheme and identifier:
            return idutils.to_url(identifier, scheme, url_scheme=url_scheme)
    except Exception:
        current_app.logger.warning(
            f"URL generation for identifier {identifier} failed.",
            exc_info=True,
        )
    return ""


def has_previewable_files(files):
    """Check if any of the files is previewable."""
    # 'splitext' inclues the dot of the file extension in the
    # extension, we have to get rid of that
    extensions = [splitext(f["key"])[-1].strip(".").lower() for f in files]
    return any([is_previewable(ext) for ext in extensions])


def has_images(files):
    """Check if any of the files are images (previewable by iiif_simple)."""
    extensions = [splitext(f["key"])[-1].strip(".").lower() for f in files]
    return any(ext in image_extensions for ext in extensions)


def order_entries(files):
    """Re-order the file entries, if an order is given."""
    order = files.get("order")
    files = files.get("entries", [])
    if order:
        files_ = files.copy()
        keys = [f["key"] for f in files]

        def get_file(key):
            idx = keys.index(key)
            keys.pop(idx)
            return files_.pop(idx)

        files = [get_file(key) for key in order]
    else:
        # sort alphabetically by filekey
        files = sorted(files, key=lambda x: x["key"].lower())

    return files


def get_scheme_label(scheme):
    """Convert backend scheme to frontend label."""
    scheme_to_label = current_app.config.get("RDM_RECORDS_IDENTIFIERS_SCHEMES", {})

    return scheme_to_label.get(scheme, {}).get("label", scheme)


def localize_number(value):
    """Format number according to locale value."""
    locale_value = current_app.config.get("BABEL_DEFAULT_LOCALE")
    number = int(value)
    return format_decimal(number, locale=locale_value)


def compact_number(value, max_value):
    """Format long numbers."""
    locale_value = current_app.config.get("BABEL_DEFAULT_LOCALE")
    number = int(value)
    decimals = 0

    if number > max_value:
        decimals = 2
    return format_compact_decimal(
        int(value), format_type="short", locale=locale_value, fraction_digits=decimals
    )


def truncate_number(value, max_value):
    """Make number compact if too long."""
    number = localize_number(value)
    if int(value) > max_value:
        number = compact_number(value, max_value=1_000_000)
    return number


def namespace_url(field):
    """Get custom field namespace url."""
    namespace_array = field.split(":")
    namespace = namespace_array[0]
    namespace_value = namespace_array[1]
    namespaces = current_app.config.get("RDM_NAMESPACES")

    if not namespaces.get(namespace):
        return None

    return namespaces[namespace] + namespace_value


def custom_fields_search(field, field_value, field_cfg=None):
    """Get custom field search url."""
    namespace_array = field.split(":")
    namespace = namespace_array[0]
    namespaces = current_app.config.get("RDM_NAMESPACES")

    if not namespaces.get(namespace):
        return None

    localised_title = (field_cfg or {}).get("locale")
    if localised_title:
        locale = get_locale()
        if not locale:
            locale = current_app.config.get("BABEL_DEFAULT_LOCALE", "en")
        # example: cern:experiments.title.en
        # the \ is necessary for the lucene syntax but produces a SyntaxWarning.
        # The r marks the string as raw and prevents the warning
        # https://docs.python.org/3/reference/lexical_analysis.html#escape-sequences
        namespace_string = r"\:".join(namespace_array) + f".{localised_title}.{locale}"
    else:
        namespace_string = r"\:".join(namespace_array)

    return url_for(
        "invenio_search_ui.search", q=f"custom_fields.{namespace_string}:{field_value}"
    )


def transform_record(record, serializer, module=None, throws=True, **kwargs):
    """Transform a record using a serializer."""
    try:
        module = module or "invenio_rdm_records.resources.serializers"
        import_str = f"{module}:{serializer}"
        serializer = obj_or_import_string(import_str)
        if serializer:
            return serializer().dump_obj(record)
        if throws:
            raise Exception("No serializer found.")
    except Exception:
        current_app.logger.error("Record transformation failed.")
        if throws:
            raise



# -*- coding: utf-8 -*-
#
# Copyright (C) 2019-2025 CERN.
# Copyright (C) 2019-2022 Northwestern University.
# Copyright (C)      2022 TU Wien.
# Copyright (C) 2023-2024 Graz University of Technology.
#
# Invenio App RDM is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.
"""Communities UI blueprints module."""

from flask import Blueprint, current_app, request
from invenio_communities.errors import CommunityDeletedError
from invenio_communities.views.ui import (
    not_found_error,
    record_permission_denied_error,
    record_tombstone_error,
)
from invenio_pidstore.errors import PIDDeletedError, PIDDoesNotExistError
from invenio_rdm_records.collections import search_app_context as c_search_app_context
from invenio_records_resources.services.errors import (
    PermissionDeniedError,
    RecordPermissionDeniedError,
)

from invenio_app_rdm.views import create_url_rule

from ..searchapp import search_app_context
from .communities import (
    communities_browse,
    communities_detail,
    communities_home,
    community_collection,
    community_static_page,
)


def _show_browse_page():
    """Whether the browse page should be visible in the menu."""
    return (
        current_app.config.get("COMMUNITIES_SHOW_BROWSE_MENU_ENTRY", False)
        and request.community["children"]["allow"]
    )


def create_ui_blueprint(app):
    """Register blueprint routes on app."""
    routes = app.config["RDM_COMMUNITIES_ROUTES"]

    blueprint = Blueprint(
        "invenio_app_rdm_communities",
        __name__,
        template_folder="../templates",
        static_folder="../static",
    )

    blueprint.add_url_rule(
        **create_url_rule(routes["community-detail"], communities_detail),
        strict_slashes=False,
    )

    blueprint.add_url_rule(
        **create_url_rule(routes["community-home"], communities_home)
    )

    blueprint.add_url_rule(
        **create_url_rule(routes["community-browse"], communities_browse)
    )

    blueprint.add_url_rule(
        **create_url_rule(routes["community-static-page"], community_static_page)
    )

    blueprint.add_url_rule(
        **create_url_rule(routes["community-collection"], community_collection)
    )
    # Register error handlers
    blueprint.register_error_handler(
        PermissionDeniedError, record_permission_denied_error
    )
    blueprint.register_error_handler(
        RecordPermissionDeniedError, record_permission_denied_error
    )
    blueprint.register_error_handler(PIDDeletedError, record_tombstone_error)
    blueprint.register_error_handler(CommunityDeletedError, record_tombstone_error)
    blueprint.register_error_handler(PIDDoesNotExistError, not_found_error)
    blueprint.register_error_handler(PIDDoesNotExistError, not_found_error)

    # Register context processor
    blueprint.app_context_processor(search_app_context)
    blueprint.app_context_processor(c_search_app_context)

    return blueprint
